/*
 * A resource offering support for accessing directories.
 *
 * Copyright (C) 2021 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA
 */

#include <ipc/cap_alloc.h>

#include "directory_resource.h"
#include "directory_object_server.h"



int DirectoryResource::expected_items()
{
    return DirectoryObject_expected_items;
}

ipc_server_handler_type DirectoryResource::handler()
{
    return (ipc_server_handler_type) handle_DirectoryObject;
}



/* Open a directory to read the directory entries via a file-like object. */

long DirectoryResource::opendir(flags_t flags, offset_t *size,
                                l4_cap_idx_t *file, object_flags_t *object_flags)
{
    /* Invoke the specialised directory opening method. */

    long err = _opendir(flags, size, file, object_flags);

    /* Handle propagated capabilities. By indicating the special status, the
       operation is first completed and then the capability is discarded. */

    if (err == IPC_MESSAGE_SENT)
    {
        complete_DirectoryObject_open(*size, *file, *object_flags);
        ipc_cap_free_um(*file);
        return IPC_MESSAGE_SENT;
    }

    return err;
}

// vim: tabstop=4 expandtab shiftwidth=4
